"""
EnamineWebServices.py - library for accessing Enamine Web Services

Use of this code is subject to the terms of the StarDrop License Agreement and
use, copying or redistribution of this file for use by other persons or
organisations is prohibited

Copyright(C) Optibrium Ltd 2019
stardrop-support@optibrium.com
"""

import urllib
import urllib2
import json
import PythonQt

BASE_URL = "https://www.enaminestore.com/api/"

class EnamineWebServices:
    def __init__(self, username, authentication_code):
        """
        Create a new instance of EnamineWebServices with the provided
        credentials

        :param username:            The username to use while accessing the API
        :param authentication_code: The password to use while accessing the API
        """
        self.username = username
        self.authentication_code = authentication_code

    def load_molecule(self, appui, molecule_id, currency):
        """
        Load a molecule with the provided Enamine molecule ID

        :param molecule_id: Enamine molecule ID e.g. EN300-190278
        :param currency:    The currency selected(either USD or EUR)

        :type molecule_id: str
        :type currency: str
        :return: Detailed information about the specified molecule
        """
        if not self.authentication_code:
            print('no authentication code')
            return None

        headers = {'Authorization': "login=" + str(self.username) + ", pass=" + str(self.authentication_code)}

        url = BASE_URL

        request_data = {"code": molecule_id, "currency": currency}

        sorted_request = sorted(request_data.items(), key=lambda val: val[0])

        request = urllib.urlencode(sorted_request)

        post_data = urllib2.Request(url + '?', data=request, headers=headers)

        try:
            resolver = urllib2.urlopen(post_data, timeout=60)
        except Exception:
            appui.show("Cannot connect to Enamine Web Services")

        json_data = resolver.read()

        result = None
        try:
            result = json.loads(json_data)
        except ValueError:
            pass

        # REAL database compounds can't be found without specifying this database to search, so if there are no
        # results returned, re-run the API call to search this database to check whether the compound is a REAL
        # database compound
        if result["data"] == None and result["result"].get("code") == 1:
            request_data = {"code": molecule_id, "currency": currency, "mode": "REAL"}

            sorted_request = sorted(request_data.items(), key=lambda val: val[0])

            request = urllib.urlencode(sorted_request)

            post_data = urllib2.Request(url + '?', data=request, headers=headers)

            try:
                resolver = urllib2.urlopen(post_data, timeout=60)
            except Exception:
                appui.show("Cannot connect to Enamine Web Services")

            json_data = resolver.read()

            result = None
            try:
                result = json.loads(json_data)
            except ValueError:
                pass

        return result

    def chemical_search(self,
                        appui,
                        smiles,
                        search_type,
                        currency,
                        similarity_index,
                        searchmode):
        """
        Perform a chemical search based on the provided input SMILES using the
        specified search strategy. The input may be a full structure or a
        fragment based on the particular search type specified.

        :param smiles:              The structure/fragment to search for
        :param search_type:         The chemical search type to use. Substructure
                                    is "IN", Similarity is "LIKE" and Exact Match
                                    is "EQ".
        :param currency:            The currency selected(either USD or EUR)
        :param similarity_index:    If using a similarity search then this is
                                    used to specify similarity in the range 0-1
        :param searchmode:          The type of search (Enamine web services allows
                                    you to search their Building blocks, screening
                                    compound or REAL compounds collections)

        :type smiles: str
        :type search_type: int
        :type currency: str
        :type similarity_index: float
        :type searchmode: str
        :return:
        """

        if not self.authentication_code:
            print('no authentication code')
            return None

        headers = {'Authorization': "login=" + str(self.username) + ", pass=" + str(self.authentication_code)}

        url = BASE_URL

        if searchmode == "Screening compounds":
            _searchmode = "SCR"
        elif searchmode == "Building blocks":
            _searchmode = "BB"
        elif searchmode == "REAL database":
            _searchmode = "REAL"

        # Searching all is not an option when doing a substructure search
        if searchmode == "Search all":
            searchmode_list = ["SCR", "BB", "REAL"]
            result = {}
            data = []

            # To search all, the API call needs to be done three times specifying the mode for the three catalogues.
            for search in searchmode_list:
                if search_type == "LIKE":
                    request_data = {"code": ('search_' + smiles + "_sim"), "currency": currency,
                                    "sim": similarity_index, "mode": search}

                else:
                    request_data = {"code": ('search_' + smiles + "_sim"), "currency": currency, "sim": '0.99',
                                    "mode": search}

                sorted_request = sorted(request_data.items(), key=lambda val: val[0])

                request = urllib.urlencode(sorted_request)

                post_data = urllib2.Request(url + "?", data=str(request), headers=headers)

                try:
                    # timout set to 60 seconds to allow for larger returned data sets
                    resolver = urllib2.urlopen(post_data, timeout=60)
                except Exception:
                    appui.show("Cannot connect to Enamine Web Services")

                json_data = resolver.read()

                partial_result = json.loads(json_data)

                # Merge the data from the different catalogues together.
                if partial_result.get('data'):
                    if not data:
                        data = partial_result.get('data')
                    else:
                        data_entry = partial_result.get('data')
                        for item in data_entry:
                            data.append(item)

            # If no compounds are found, return the last result to get the error code.
            if not data:
                result = partial_result
            else:
                result["data"] = data

        else:
            if search_type == "IN":
                request_data = {"code": ('search_' + smiles + "_sub"), "currency": currency, "mode": _searchmode}

            elif search_type == "LIKE":
                request_data = {"code": ('search_' + smiles + "_sim"), "currency": currency, "sim": similarity_index, "mode": _searchmode}

            else:
                request_data = {"code": ('search_' + smiles + "_sim"), "currency": currency, "sim": '0.99', "mode": _searchmode}

            sorted_request = sorted(request_data.items(), key=lambda val: val[0])

            request = urllib.urlencode(sorted_request)

            post_data = urllib2.Request(url + "?", data=str(request), headers=headers)

            try:
                # timout set to 60 seconds to allow for larger returned data sets
                resolver = urllib2.urlopen(post_data, timeout=60)
            except Exception:
                appui.show("Cannot connect to Enamine Web Services")

            json_data = resolver.read()

            result = json.loads(json_data)

        return result
